#include <string.h>
#include "stuff.h"
#include "winnt.h"
#include "pe.h"
#include <stdio.h>

int get_sections_info(PeHeader *PE)
    {
    int i;
    IMAGE_FILE_HEADER     *pe_header;
    IMAGE_SECTION_HEADER  *p_section;

    pe_header=PE->GetFileHeader();
if(pe_header==NULL)return -1;
    printf("\nSECTIONS\n"\
	     "========\n");
    printf("Num  Name     Characteristics VirtualAddress SizeOfRawData VirtualSize\n");
    for(i=0;i<pe_header->NumberOfSections;i++){
       p_section=PE->GetSectionHeader(i);
       if(!p_section)continue;
       printf("%2.2u   %-8.8s ",i,p_section->Name);
       printf("%15.8x ",p_section->Characteristics);
       printf("%14.8x ",p_section->VirtualAddress);
       printf("%13.8x ",p_section->SizeOfRawData);
       printf("%11.8x \n",p_section->Misc.VirtualSize);}
    return 0;
    }

int get_image_info(PeHeader *PE)
    {
IMAGE_OPTIONAL_HEADER *pe_opt_header;
IMAGE_FILE_HEADER *pe_header;
pe_opt_header=PE->GetOptionalHeader();
pe_header=PE->GetFileHeader();

if(pe_opt_header==NULL||pe_header==NULL)return -1;
    printf("\nIMAGE_FILE_HEADER\n"\
             "=================\n");
    printf("Signature            %8.8x\t",pe_header->Signature);            
    printf("Machine              %4.4x\n",pe_header->Machine);              
    printf("NumberOfSections     %4.4x\t",pe_header->NumberOfSections);     
    printf("TimeDateStamp        %8.8x\n",pe_header->TimeDateStamp);        
    printf("PointerToSymbolTable %8.8x\t",pe_header->PointerToSymbolTable); 
    printf("NumberOfSymbols      %8.8x\n",pe_header->NumberOfSymbols);      
    printf("SizeOfOptionalHeader %4.4x\t",pe_header->SizeOfOptionalHeader); 
    printf("Characteristics      %4.4x\n",pe_header->Characteristics);      
            

    printf("\nIMAGE_OPTIONAL_HEADER\n"\
             "=====================\n");
    printf("MajorLinkerVersion          %2.2x\t\t",pe_opt_header->MajorLinkerVersion);
    printf("MinorLinkerVersion          %2.2x\n",pe_opt_header->MinorLinkerVersion);
    printf("SizeOfInitializedData       %8.8x\t",pe_opt_header->SizeOfInitializedData);
    printf("SizeOfUninitializedData     %8.8x\n",pe_opt_header->SizeOfUninitializedData);
    printf("AddressOfEntryPoint         %8.8x\t",pe_opt_header->AddressOfEntryPoint);
    printf("BaseOfCode                  %8.8x\n",pe_opt_header->BaseOfCode);
    printf("BaseOfData                  %8.8x\t",pe_opt_header->BaseOfData);
    printf("SizeOfCode                  %8.8x\n",pe_opt_header->SizeOfCode);
    printf("ImageBase                   %8.8x\t",pe_opt_header->ImageBase);
    printf("MajorOperatingSystemVersion %4.4x\n",pe_opt_header->MajorOperatingSystemVersion);
    printf("MinorOperatingSystemVersion %4.4x\t",pe_opt_header->MinorOperatingSystemVersion);
    printf("MajorImageVersion           %4.4x\n",pe_opt_header->MajorImageVersion);
    printf("MinorImageVersion           %4.4x\t",pe_opt_header->MinorImageVersion);
    printf("MajorSubsystemVersion       %4.4x\n",pe_opt_header->MajorSubsystemVersion);
    printf("MinorSubsystemVersion       %4.4x\t",pe_opt_header->MinorSubsystemVersion);
    printf("CheckSum                    %8.8x\n",pe_opt_header->CheckSum);
    printf("Subsystem                   %4.4x\t",pe_opt_header->Subsystem);
    printf("DllCharacteristics          %4.4x\n",pe_opt_header->DllCharacteristics);
    printf("SizeOfStackReserve          %4.4x\t",pe_opt_header->SizeOfStackReserve);
    printf("SizeOfStackCommit           %4.4x\n",pe_opt_header->SizeOfStackCommit);
    printf("SizeOfHeapReserve           %4.4x\t",pe_opt_header->SizeOfHeapReserve);
    printf("SizeOfHeapCommit            %4.4x\n",pe_opt_header->SizeOfHeapCommit);
    printf("LoaderFlags                 %4.4x\t",pe_opt_header->LoaderFlags);
    printf("NumberOfRvaAndSizes         %4.4x\n",pe_opt_header->NumberOfRvaAndSizes);
    return 0;
    }

int get_directory_info(PeHeader *PE)
{
IMAGE_OPTIONAL_HEADER *OptHeader;

int i,sec_num;
char sec_name[9];
sec_name[8]=0;
uint32 sec_offs;

static char *spec_names[IMAGE_NUMBEROF_DIRECTORY_ENTRIES]={"Export","Import","Resource","Exception","Security","BaseReloc","Debug","Copyright","GlobalPtr","Tls","LoadConfig","BoundImport","Iat","Reserved","Reserved","Reserved"};

IMAGE_SECTION_HEADER  *p_section;
OptHeader=PE->GetOptionalHeader();
char tmp[255];

if(OptHeader==NULL)return -1;
printf("\nDIRECTORIES\n"\
         "===========\n");
printf("Num Directory        RVA      Size     Section      SecOffs\n");
for(i=0;i<IMAGE_NUMBEROF_DIRECTORY_ENTRIES;i++)
	{
	if(PE->Rva2Lseek(OptHeader->DataDirectory[i].VirtualAddress,&sec_num)!=-1){
		p_section=PE->GetSectionHeader(sec_num);
		strcpy(sec_name,(char*)p_section->Name);
		sec_offs=OptHeader->DataDirectory[i].VirtualAddress-p_section->VirtualAddress;
		sprintf(tmp,"%2.2u %+8.8s  %8.8x",sec_num,sec_name,sec_offs);
	}else {
		sprintf(tmp,"<no section>");
		}
		printf("%-2.2u  %-16.16s %-8.8x %-8.8x %s\n",i,spec_names[i],OptHeader->DataDirectory[i].VirtualAddress,OptHeader->DataDirectory[i].Size,tmp);
	}
return 0;
}

int PrintInfo(char* fname,int extend)
{
PeHeader PE;

IMAGE_OPTIONAL_HEADER *OptHeader;
IMAGE_FILE_HEADER     *FileHeader;
IMAGE_SECTION_HEADER  *SecHeader;

if (PE.LoadHeader(fname)==-1){
                              printf("ERROR:%s\n",PE.GetLastErrorText());
                              return -1;
                                };

OptHeader=PE.GetOptionalHeader();
FileHeader=PE.GetFileHeader();

if(OptHeader==NULL||FileHeader==NULL)return -1;

get_image_info(&PE);
if(extend){
	get_sections_info(&PE);
	get_directory_info(&PE);
}
return 0;
}