#include <malloc.h>
#include <memory.h>
#include <io.h>
#include <fcntl.h>
#include "pe.h"
#include <stdio.h>
#include "stuff.h"

/*
 * Just errors messages..
 */
 
char* PeHeader::_PeHeaderErrorsMsg[4]={"Undefened error!",
                    "File I/O error!",
                    "Memory error!",
                    "PE format error!"};   

//==========================================================================

PeHeader::PeHeader()
{
SectionHeaders=NULL;
PeHeaderError=UNDEF;
}

//==========================================================================

PeHeader::~PeHeader()
{
if(SectionHeaders)free(SectionHeaders);
}

//==========================================================================

PeHeader::LoadHeader(const char* fname)
{
uint			i;
int 			PeHan;
uint32 			PeHeaderPos;

PeHan=open(fname,O_RDWR|O_BINARY);
if(PeHan==-1){PeHeaderError=FILE_IO;return -1;}

/* Reading MZ header. 
 * Get PE header position
 */

if(SectionHeaders)delete SectionHeaders;
memset (&MzHeader, 0, sizeof(IMAGE_DOS_HEADER));
if (lseek(PeHan, 0, SEEK_SET)==-1) {PeHeaderError=FILE_IO;return -1;}
if (read(PeHan, &MzHeader, sizeof(IMAGE_DOS_HEADER))==-1)
                                   {PeHeaderError=FILE_IO;return -1;}
if ((MzHeader.e_magic)!=0x5a4d)   {PeHeaderError=FILE_IO;return -1;}
PeHeaderPos=MzHeader.e_lfanew;

/* 
 * Reading FileHeader
 */

if (lseek(PeHan, PeHeaderPos, SEEK_SET)==-1)
                              {PeHeaderError=FILE_IO;return -1;}
if (read(PeHan, &FileHeader, sizeof(IMAGE_FILE_HEADER))==-1)
   			      {PeHeaderError=FILE_IO;return -1;}
if (FileHeader.Signature!=0x00004550)
                              {PeHeaderError=PE_FORMAT;return -1;}
if (FileHeader.SizeOfOptionalHeader > sizeof(IMAGE_OPTIONAL_HEADER))
                              {PeHeaderError=PE_FORMAT;return -1;}
/*                
 * Reading optional header
 */

if (read(PeHan, &OptionalHeader, FileHeader.SizeOfOptionalHeader)==-1)return -1;
SectionHeaders=(IMAGE_SECTION_HEADER*)malloc(FileHeader.NumberOfSections*
				sizeof(IMAGE_SECTION_HEADER));
/*
 * Reading sections
 */

if (!SectionHeaders)   {PeHeaderError=MEMORY;return -1;}
for (i=0;i<FileHeader.NumberOfSections;i++)
      if (read(PeHan, &SectionHeaders[i],
          sizeof(IMAGE_SECTION_HEADER))==-1) {PeHeaderError=FILE_IO;return -1;}
RealSizeOfHeaders=tell(PeHan);
 if (RealSizeOfHeaders==-1) {PeHeaderError=FILE_IO;return -1; }
return close(PeHan);
}

//==========================================================================

uint32 PeHeader::Rva2Lseek (uint32 Rva,int *num)
{
uint 		i;
int k;
if(num==NULL)num=&k;

/* 
 * Looking in section headers if rva lies in it
 */

for (i=0;i<=FileHeader.NumberOfSections;i++){
       if (i==FileHeader.NumberOfSections)return -1;
       if(
          (
          SectionHeaders[i].VirtualAddress<=(uint32)Rva
          )
          &&
          (
           SectionHeaders[i].VirtualAddress+
           SectionHeaders[i].SizeOfRawData > (uint32)Rva
          )
         )
                                                     break;
                                            }
*num=i;
return (Rva-SectionHeaders[i].VirtualAddress+
            SectionHeaders[i].PointerToRawData);

}

//==========================================================================

uint32  PeHeader::Tell2Rva(uint32  Pos,int* num)
{
uint i;
int k;
if(num==NULL)num=&k;

/* 
 * Looking in section headers if Raw pointer points in it
 */

for (i=0;i<=FileHeader.NumberOfSections;i++){
          if (i==FileHeader.NumberOfSections)return -1;
          if (
               Pos>=SectionHeaders[i].PointerToRawData
               &&
               Pos<SectionHeaders[i].PointerToRawData
               +SectionHeaders[i].SizeOfRawData
             )
             break;
                                              }
*num=i;
return (SectionHeaders[i].VirtualAddress+
        Pos-SectionHeaders[i].PointerToRawData);
}

//==========================================================================

IMAGE_DATA_DIRECTORY *PeHeader::GetDirectory(uint DirNum)
{
if (DirNum>=IMAGE_NUMBEROF_DIRECTORY_ENTRIES)return NULL;
return &OptionalHeader.DataDirectory[DirNum];
}

//==========================================================================

IMAGE_SECTION_HEADER *PeHeader::GetSectionHeader(uint SecNum)
{
if (SecNum>=FileHeader.NumberOfSections)return NULL;
return &SectionHeaders[SecNum];
}

//==========================================================================

IMAGE_FILE_HEADER *PeHeader::GetFileHeader()
{
return &FileHeader;
}

//==========================================================================

IMAGE_OPTIONAL_HEADER *PeHeader::GetOptionalHeader()
{
return &OptionalHeader;
}

//==========================================================================

int PeHeader::SaveHeader(const char *fname)
{
uint i;
int PeHan=open(fname,O_RDWR|O_BINARY);
if (PeHan==-1)
					{PeHeaderError=FILE_IO;return -1;}
if (lseek(PeHan,MzHeader.e_lfanew,SEEK_SET)==-1) 
					{PeHeaderError=FILE_IO;return -1;}
if (write(PeHan,&FileHeader,sizeof(IMAGE_FILE_HEADER))==-1)
					{PeHeaderError=FILE_IO;return -1;}
if (write(PeHan,&OptionalHeader,FileHeader.SizeOfOptionalHeader)==-1)
					{PeHeaderError=FILE_IO;return -1;}
for (i=0;i<FileHeader.NumberOfSections;i++)
    if (write(PeHan,&SectionHeaders[i],sizeof(IMAGE_SECTION_HEADER))==-1)
					{PeHeaderError=FILE_IO;return -1;}
return close(PeHan);
}

//=========================================================================

int PeHeader::PushSectionHeader(const IMAGE_SECTION_HEADER *SecHeader)
{
SectionHeaders=(IMAGE_SECTION_HEADER*)
               realloc(SectionHeaders,sizeof(IMAGE_SECTION_HEADER)*
		       (FileHeader.NumberOfSections+1));
if(!SectionHeaders) {PeHeaderError=MEMORY;return -1;}
memcpy(&SectionHeaders[FileHeader.NumberOfSections++],SecHeader,
			sizeof(IMAGE_SECTION_HEADER));
return 0;
}       

//========================================================================

int PeHeader::PopSectionHeader()
{
SectionHeaders=(IMAGE_SECTION_HEADER*)
	        realloc(SectionHeaders,sizeof(IMAGE_SECTION_HEADER)*
		       (--FileHeader.NumberOfSections));
if(!SectionHeaders) {PeHeaderError=MEMORY;return -1;}
return 0;
}       

//========================================================================
uint32 PeHeader::GetRealSizeOfHeaders()
{
return RealSizeOfHeaders;
}
